/*
 *
 * Copyright (C) 2006-2007  imaxin|software
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * */

#include <iostream>
#include <string>

#include "Disambiguator.h"

using namespace std;

Disambiguator::Disambiguator(
								const string path_tokenizer,
								const string path_splitter,
								const string path_quanties,
								const string path_locutions,
								const string path_suffix,
								const string path_probability,
								const string path_dictionary,
								const string path_np,
								const string path_punctuation,
								const string path_tagger,
								const string language
							) {

	_tk		= new tokenizer(path_tokenizer);
	_sp		= new splitter(path_splitter);
	_opt 	= new maco_options(language);
	
	_opt->QuantitiesDetection 	= false;
	_opt->SuffixAnalysis 		= true;
	_opt->MultiwordsDetection 	= true;
	_opt->NumbersDetection 		= true;
	_opt->PunctuationDetection 	= true;
	_opt->DatesDetection 		= true;
	_opt->QuantitiesDetection 	= true;
	_opt->DictionarySearch 		= true;
	_opt->ProbabilityAssignment = true;
	_opt->NERecognition 		= true;

	_opt->QuantitiesFile 	= path_quanties;
	_opt->LocutionsFile		= path_locutions;
	_opt->SuffixFile		= path_suffix;
	_opt->ProbabilityFile	= path_probability;
	_opt->DictionaryFile	= path_dictionary;
	_opt->NPdataFile		= path_np;
	_opt->PunctuationFile	= path_punctuation;

	_morfo 	= new maco(*_opt);
	_tg 	= new hmm_tagger(language, path_tagger, true, true);
}

Disambiguator::~Disambiguator () {
	delete _tk;
	delete _sp;
	delete _opt;
	delete _morfo;
	delete _tg;	
}

void Disambiguator::run(Phrase *ora) {
	list<word> 		lw = _tk->tokenize(ora->getOriginalString());
	list<sentence> 	ls = _sp->split(lw, true);
	
	_morfo->analyze(ls);
	_tg->analyze(ls);
	ora->setDisambiguated(ls);
}
