/*
 *
 * Copyright (C) 2006-2007  imaxin|software
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * */

#include <iostream>
#include <string>

#include "Phrase.h"

using namespace std;

Phrase::Phrase() {
}

Phrase::Phrase(const string ora) {
	_ora	= ora;
	_status = 0;
}

Phrase::Phrase(Phrase &p) {
	_status = p._status;
	_ora = p._ora;
	_list = p._list;
}

Phrase & Phrase::operator = (const Phrase &p) {
	*this = p;
	return *this;
}

Phrase::~Phrase () {
	_list.clear();
}

void Phrase::setNotDisambiguated(const string ora) {
	_ora = ora;
}

void Phrase::setDisambiguated(list<sentence> ls) {
	list<sentence>::const_iterator 	s;
	sentence::const_iterator 	w;

	for (s = ls.begin(); s != ls.end(); s++)
		for (w = s->begin(); w != s->end(); w++)
			_list.push_back(*w);
}

string Phrase::getOriginalString(void) {
	return _ora;
}

string Phrase::getParolesConcat(void) {
	string buffer("");
	int i = 0;
	int size = _list.size();

	for (i = 0; i < size; i++)
		if (i == size - 1)
			buffer += _list.at(i).get_parole();
		else
			buffer += _list.at(i).get_parole() + "+";

	return buffer;
}

string Phrase::getAnalysisConcat() {
	string buffer("");
	int i = 0;
	int size = _list.size();

	for (i = 0; i < size; i++)
		if (i == size - 1)
			buffer += _list.at(i).get_parole() + "_" + _list.at(i).get_lemma();
		else
			buffer += _list.at(i).get_parole() + "_" + _list.at(i).get_lemma() + "+";

	return buffer;
}

vector<string> Phrase::getLemmaCollection() {
	vector<string> v_lemas;
	int i = 0;
	int size = _list.size();

	for (i = 0; i < size; i++)
		v_lemas.push_back(_list.at(i).get_lemma());

	return v_lemas;
}

vector<string> Phrase::getLemmaCollectionFilteredByName() {
	vector<string> v_lemas;
	int i = 0;
	int size = _list.size();

	for (i = 0; i < size; i++)
		if (pcrecpp::RE("^N.*$").FullMatch(_list.at(i).get_parole()))
			v_lemas.push_back(_list.at(i).get_lemma());

	return v_lemas;
}

string Phrase::getWord(int index) {
	return _list.at(index).get_form();
}

string Phrase::getLemma(int index) {
	return _list.at(index).get_lemma();
}

string Phrase::getParole(int index) {
	return _list.at(index).get_parole();
}

int Phrase::getSize() {
	return _list.size();
}

string Phrase::getSpan(int index) {
	long pos_ini = 0;
	long pos_end = 0;

	if (index < _list.size()) {
		pos_ini = _list.at(index).get_span_start();
		pos_end = _list.at(index).get_span_finish();
	}
	else if (index == _list.size()) {
		pos_ini = _list.at(index - 1).get_span_start();
		pos_end = _list.at(index - 1).get_span_finish();
	}

	string buffer = _ora.substr(pos_ini, pos_end - pos_ini);
	return buffer;
}

string Phrase::getSpan(int init, int end) {
	int i = init;
	vector<string> list;
	string buffer = "";

	for (i = init; i <= end; i++) {
		buffer = this->getSpan(i);
		if (Util::findStringInVector(buffer, list, 1) < 0)
			list.push_back(buffer);
	}

	int size = list.size();
	buffer = "";

	for (i = 0; i < size; i++)
		buffer += list.at(i) + " ";

	StringUtil::Trim(buffer);

	return buffer;
}
