/*
 *
 * Copyright (C) 2006-2007  imaxin|software
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * */

#include <iostream>
#include <string>

#include "Solution.h"
#ifndef STRINGUTIL_H
#define STRINGUTIL_H
#include "StringUtil.h"
#endif
#ifndef UTIL_H
#define UTIL_H
#include "Util.h"
#endif
using namespace std;

#ifdef DEBUG
#define DPRINT(var)		cout << var << std::endl
#define DPRINT(var, val) 	cout << var << ": " << val << std::endl
#else
#define DPRINT(var)
#define DPRINT(var, val)
#endif
#define MAX_SIZE_DETL 0x00000050
#define MAX_SIZE_DETC 0x00000070
const char *_detl[] = {	"o",		"os",		"a",		"as",		/*0x00000000*/
						"este",		"estes",	"esta",		"estas",	/*0x00000004*/
						"aquel",	"aqueles",	"aquela",	"aquelas",	/*0x00000008*/
						"ese",		"eses",		"esa",		"esas",		/*0x0000000C*/
						"un",		"uns",		"unha",		"unhas",	/*0x00000010*/
						"outro",	"outros",	"outra",	"outras",	/*0x00000014*/
						"algn",	"algns",	"algunha",	"algunhas",	/*0x00000018*/
						"seu",		"seus",		"sa",		"sas",		/*0x0000001C*/
						"teu",		"teus",		"ta",		"tas",		/*0x00000020*/
						"meu",		"meus",		"mia",		"mias",	/*0x00000024*/
						"noso",		"nosos",	"nosa",		"nosas",	/*0x00000028*/
						"voso",		"vosos",	"vosa",		"vosas",	/*0x0000002C*/
						"ningn",	"ningns",	"ningunha",	"ningunhas",	/*0x00000030*/
						"vario",	"varios",	"varia",	"varias",	/*0x00000034*/
						"canto",	"cantos",	"canta",	"cantas",	/*0x00000038*/
						"propio",	"propios",	"propia",	"propias",	/*0x0000003C*/
						"certo",	"certos",	"certa",	"certas",	/*0x00000040*/
						"pouco",	"poucos",	"pouca",	"poucas",	/*0x00000044*/
						"mesmo",	"mesmos",	"mesma",	"mesmas",	/*0x00000048*/
						"tanto",	"tantos",	"tanta",	"tantas"	/*0x0000004C*/
						};

const char *_detc[] = {	"do",		"dos",		"da",		"das",		/*0x00000000*/
						"deste",	"destes",	"desta",	"destas",
						"daquel",	"daqueles",	"daquela",	"daquelas",
						"dese",		"deses",	"desa",		"desas",
						"dun",		"duns",		"dunha",	"dunhas",
						"doutro",	"doutros",	"doutra",	"doutras",
						"dalgn",	"dalgns",	"dalgunha",	"dalgunhas",

						"no",		"nos",		"na",		"nas",		/*0x0000001C*/
						"neste",	"nestes",	"nesta",	"nestas",
						"naquel",	"naqueles",	"naquela",	"naquelas",
						"nese",		"neses",	"nesa",		"nesas",
						"nun",		"nuns",		"nunha",	"nunhas",
						"noutro",	"noutros",	"noutra",	"noutras",
						"nalgn",	"nalgns",	"nalgunha",	"nalgunhas",

						"co",		"cos",		"coa",		"coas",		/*0x00000038*/
						"",			"",			"",			"",
						"",			"",			"",			"",
						"",			"",			"",			"",
						"cun",		"cuns",		"cunha",	"cunhas",
						"",			"",			"",			"",
						"",			"",			"",			"",

						"ao",		"aos",		"",		"s",		/*0x00000054*/
						"",			"",			"",			"",
						"",			"",			"",			"",
						"",			"",			"",			"",
						"",			"",			"",			"",
						"",			"",			"",			"",
						"",			"",			"",			""
						};

Solution::Solution(PhraseStructure ora) {
	_ora = new PhraseStructure(ora);
}

Solution::~Solution() {
	delete _ora;
}

int Solution::buildGrammarBetweenArticleAndName(int pos_error,
												vector<string> v_sugg,
												vector<string> v_sugg_tagged,
												vector<string> *v_candidates,
												string *error_replace
												) {


	int error = -1;
	int type = 0;
	int pos = 0;

	if (Util::setPosErrorAndType(pos_error, &pos, &type)) {
		if (pos > -1) {
			if (_ora->isDeterminerMaleThePreviousTag(pos)) {

					if ( (error = this->rebuildErrorWithDeterminerForThePreviousTag(pos, type, error_replace)) < 0)
						return error;

					int case_to_use = Util::isUpper(*error_replace);

					error = this->rebuildCandidatesWithDeterminerForThePreviousTag(	pos,
																					v_sugg,
																					v_sugg_tagged,
																					v_candidates,
																					type,
																					case_to_use
																					);

					return error;
			}
			else {
				error = this->buildWithoutGrammarBetweenArticleAndName(	pos_error,
																		v_sugg,
																		v_candidates,
																		error_replace);
				return error;
			}

		}
		else
			return error;
	}
	else
		return error;

}

int Solution::rebuildCandidatesWithDeterminerForThePreviousTag(	int pos_error,
																vector<string> v_sugg,
																vector<string> v_sugg_tagged,
																vector<string> *v_candidates,
																int type,
																int case_type
															) {

	int error = -1;

	if (v_candidates) {

		if (type == ERROR_TYPE_INVARIABLE_FIRST) {
			*v_candidates = Util::convertCase(v_sugg, case_type);
			return v_candidates->size();
		}
		else if	(type == ERROR_TYPE_SIMPLE) {
			if (!_ora->isContractionThePrePreviousTag(pos_error))
				error = this->createGrammarWithoutContraction(	pos_error,
																v_sugg,
																v_sugg_tagged,
																v_candidates,
																ERROR_TYPE_SIMPLE,
																case_type);
			else
				error = this->createGrammarWithContraction(	pos_error,
															v_sugg,
															v_sugg_tagged,
															v_candidates,
															case_type);

			return error;
		}
		else if (type == ERROR_TYPE_INVARIABLE) {
			error = this->createGrammarWithoutContraction(	pos_error,
															v_sugg,
															v_sugg_tagged,
															v_candidates,
															ERROR_TYPE_INVARIABLE,
															case_type);
			return error;
		}

	}
	else
		return error;
}

int Solution::createGrammarWithContraction(	int pos_error,
											vector<string> v_sugg,
											vector<string> v_sugg_tagged,
											vector<string> *v_candidates,
											int case_type
											) {

	string lemma_article	= _ora->getLemma(pos_error - 1);
	string preposition		= _ora->getLemma(pos_error - 2);
	int size				= v_sugg_tagged.size();
	int i;

	for (i = 0; i < size; i++) {
		string tag_for_name		= v_sugg_tagged.at(i);
		string word_candidate	= v_sugg.at(i);
		string det_candidate	= "";

		if (!this->getCompleteContractionWithPreposition(preposition, lemma_article, tag_for_name, &det_candidate))
			return -1;
		string buffer = det_candidate + " " + word_candidate;
		buffer = Util::convertCase(buffer, case_type);
		v_candidates->push_back(buffer);
	}

	return v_candidates->size();
}

int Solution::createGrammarWithoutContraction(	int pos_error,
												vector<string> v_sugg,
												vector<string> v_sugg_tagged,
												vector<string> *v_candidates,
												int type,
												int case_type
												) {

	string lemma_article	= _ora->getLemma(pos_error - 1);
	int size				= v_sugg_tagged.size();
	int i = 0;

	for (i = 0; i < size; i++) {
		string tag_for_name		= v_sugg_tagged.at(i);
		string det_candidate	= "";
		string word_candidate	= v_sugg.at(i);

		if (!this->getCompleteArticleWithoutPreposition(lemma_article, tag_for_name, &det_candidate))
			return -1;

		if (type == ERROR_TYPE_SIMPLE) {
			string buffer = det_candidate + " " + word_candidate;
			buffer = Util::convertCase(buffer, case_type);
			v_candidates->push_back(buffer);

		}
		else if (type == ERROR_TYPE_INVARIABLE) {
			string det_candidate_fem = "";

			if (!this->getCompleteArticleWithoutPrepositionInFemale(lemma_article, tag_for_name, &det_candidate_fem))
				return -1;

			string buffer = this->replaceExpression(	word_candidate,
														det_candidate,
														det_candidate_fem);
			buffer = Util::convertCase(buffer, case_type);
			v_candidates->push_back(buffer);
		}

	}

	return v_candidates->size();
}

int Solution::rebuildErrorWithDeterminerForThePreviousTag(int pos_error, int type, string *error_replace) {
	if (error_replace) {
		if (type == ERROR_TYPE_INVARIABLE_FIRST) {
			if (!this->rebuildErrorForPrepositionArticleName(pos_error, error_replace))
				return -1;
		}
		else if (type == ERROR_TYPE_SIMPLE) {
			if (_ora->isContractionThePrePreviousTag(pos_error)) {
				if (!this->rebuildErrorForPrepositionArticleName(pos_error, error_replace))
					return -1;
			}
			else {
				if (!this->rebuildErrorForArticleName(pos_error, error_replace))
					return -1;
			}
		}
		else if (type == ERROR_TYPE_INVARIABLE) {
			if (!this->rebuildErrorForArticleName(pos_error, error_replace))
				return -1;
		}

		return 1;
	}
	else
		return -1;

}

int Solution::rebuildErrorForPrepositionArticleName(int pos_error, string *error_replace) {

	if (error_replace)
		*error_replace = _ora->getSpan(pos_error - 2, pos_error);
	else
		return 0;

	return 1;
}

int Solution::rebuildErrorForArticleName(int pos_error, string *error_replace) {
	if (error_replace)
		*error_replace = _ora->getSpan(pos_error - 1, pos_error);
	else
		return 0;

	return 1;
}

int Solution::rebuildErrorWithContractionWithArticleAndName(int pos_error, string *error_replace) {
	if (error_replace) {
		*error_replace = _ora->getWord(pos_error + 2);
		return 1;
	}
	else
		return -1;
}

int Solution::rebuildErrorForOnlyName(int pos_error, string *error_replace) {
	if (error_replace)
		*error_replace = _ora->getSpan(pos_error);
	else
		return 0;

	return 1;
}

int Solution::getTagPosition(string tag) {
	if		(pcrecpp::RE(".*MS.*").FullMatch(tag) || pcrecpp::RE("^NCCS.*").FullMatch(tag))
		return 0;
	else if (pcrecpp::RE(".*MP.*").FullMatch(tag) || pcrecpp::RE("^NCCP.*").FullMatch(tag))
		return 1;
	else if (pcrecpp::RE(".*FS.*").FullMatch(tag))
		return 2;
	else if (pcrecpp::RE(".*FP.*").FullMatch(tag))
		return 3;
	else
		return -1;

}

int Solution::getWordPosition(string word) {
	if		(word == "o")
		return 0x00000000;
	else if (word == "este")
		return 0x00000004;
	else if (word == "aquel")
		return 0x00000008;
	else if (word == "ese")
		return 0x0000000C;
	else if (word == "un")
		return 0x00000010;
	else if (word == "outro")
		return 0x00000014;
	else if (word == "algn")
		return 0x00000018;
	else if (word == "seu")
		return 0x0000001C;
	else if (word == "teu")
		return 0x00000020;
	else if (word == "meu")
		return 0x00000024;
	else if (word == "noso")
		return 0x00000028;
	else if (word == "voso")
		return 0x0000002C;
	else if (word == "ningn")
		return 0x00000030;
	else if (word == "varios")
		return 0x00000034;
	else if (word == "canto")
		return 0x00000038;
	else if (word == "propio")
		return 0x0000003C;
	else if (word == "certo")
		return 0x00000040;
	else if (word == "pouco")
		return 0x00000044;
	else if (word == "mesmo")
		return 0x00000048;
	else if (word == "tanto")
		return 0x0000004C;
	else
		return -1;
}

int Solution::getPrepositionPosition(string pre) {
	if		(pre == "de")
		return 0x00000000;
	else if (pre == "en")
		return 0x0000001C;
	else if (pre == "con")
		return 0x00000038;
	else if (pre == "a")
		return 0x00000054;
	else
		return -1;
}

string Solution::replaceExpression(string word, string article, string alt_article) {
	string buffer;

	buffer = Util::substitute(	word,
								"_/_",
								article + "/" + alt_article
								);

	buffer = Util::substitute(	buffer,
								"_ou_",
								article + " ou " + alt_article
								);

	buffer = Util::substitute(	buffer,
								"_e_",
								article + " e " + alt_article
								);

	return buffer;
}

int Solution::getCompleteContractionWithPreposition(string preposition, string article, string tag_for_name, string *candidate) {

	if (candidate) {
		int offset_word			= this->getWordPosition(article);
		int offset_pre			= this->getPrepositionPosition(preposition);
		int offset_tag			= this->getTagPosition(tag_for_name);

		if (offset_word > -1 &&
			offset_tag	> -1 &&
			offset_pre	> -1) {

			string det_candidate	= _detc[offset_word + offset_tag + offset_pre];

			*candidate = det_candidate;

			return 1;
		}
		else
			return 0;
	}
	else
		return 0;
}

int Solution::getCompleteArticleWithoutPreposition(	string article,
													string tag_for_name,
													string *candidate) {
	int error = -1;

	if (candidate) {
		int offset_word = this->getWordPosition(article);
		int offset_tag	= this->getTagPosition(tag_for_name);

		if (offset_word + offset_tag > -1 &&
			offset_word + offset_tag < MAX_SIZE_DETL)
			*candidate	= _detl[offset_word + offset_tag];
		else
			return error;
	}
	else
		return error;

	return 1;
}

int Solution::getCompleteArticleWithoutPrepositionInFemale(	string article,
															string tag_for_name,
															string *candidate) {
	int error = -1;

	if (candidate) {
		int offset_word = this->getWordPosition(article);
		int offset_tag	= this->getTagPosition(tag_for_name);

		if (offset_word + offset_tag > -1 &&
			offset_word + offset_tag < MAX_SIZE_DETL - 2)
			*candidate	= _detl[offset_word + offset_tag + 2];
		else
			return error;
	}
	else
		return error;

	return 1;
}

int Solution::buildWithoutGrammarBetweenArticleAndName(	int pos_error,
														vector<string> v_sugg,
														vector<string> *v_candidates,
														string *error_replace) {
	int error = -1;
	int type = 0;

	if (Util::setPosErrorAndType(pos_error, &pos_error, &type)) {
		if (pos_error > -1) {
			if (!this->rebuildErrorForOnlyName(pos_error, error_replace))
				return error;

			if (v_candidates) {
				int case_to_use = Util::isUpper(*error_replace);

				*v_candidates = Util::convertCase(v_sugg, case_to_use);
			}

			return pos_error;
		}
		else
			return error;
	}
	else return error;
}
